-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with Mast.Scheduling_Servers,Mast.Operations,
   Indexed_Lists, Ada.Text_IO;

package Mast.Drivers is

   --------------------
   -- Abstract Driver
   --------------------

   type Driver is abstract tagged private;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Drv : in out Driver;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   type Driver_Ref is access Driver'Class;

   function Clone
     (Drv : Driver)
     return  Driver_Ref is abstract;

   package Lists is new Indexed_Lists
     (Element => Driver_Ref,
      "=" => "=");

   function Clone
     (The_List : in Lists.List)
     return Lists.List;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      The_List : in out Lists.List;
      Indentation : Positive);

   --------------------
   -- Packet_Driver
   --------------------

   type Packet_Driver is new Driver with private;

   procedure Set_Packet_Server
     (The_Driver : in out Packet_Driver;
      The_Packet_Server : Scheduling_Servers.Scheduling_Server_Ref);
   function Packet_Server
     (The_Driver : Packet_Driver)
     return Scheduling_Servers.Scheduling_Server_Ref;

   procedure Set_Packet_Send_Operation
     (The_Driver : in out Packet_Driver;
      The_Packet_Send_Operation : Operations.Simple_Operation_Ref);
   function Packet_Send_Operation
     (The_Driver : Packet_Driver)
     return Operations.Simple_Operation_Ref;

   procedure Set_Packet_Receive_Operation
     (The_Driver : in out Packet_Driver;
      The_Packet_Receive_Operation : Operations.Simple_Operation_Ref);
   function Packet_Receive_Operation
     (The_Driver : Packet_Driver)
     return Operations.Simple_Operation_Ref;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Drv : in out Packet_Driver;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   function Clone
     (Drv : Packet_Driver)
     return  Driver_Ref;

   -----------------------------
   -- Character_Packet_Driver --
   -----------------------------

   type Character_Packet_Driver is new Packet_Driver with private;

   procedure Set_Character_Server
     (The_Driver : in out Character_Packet_Driver;
      The_Character_Server : Scheduling_Servers.Scheduling_Server_Ref);
   function Character_Server
     (The_Driver : Character_Packet_Driver)
     return Scheduling_Servers.Scheduling_Server_Ref;

   procedure Set_Character_Send_Operation
     (The_Driver : in out Character_Packet_Driver;
      The_Character_Send_Operation : Operations.Simple_Operation_Ref);
   function Character_Send_Operation
     (The_Driver : Character_Packet_Driver)
     return Operations.Simple_Operation_Ref;

   procedure Set_Character_Receive_Operation
     (The_Driver : in out Character_Packet_Driver;
      The_Character_Receive_Operation : Operations.Simple_Operation_Ref);
   function Character_Receive_Operation
     (The_Driver : Character_Packet_Driver)
     return Operations.Simple_Operation_Ref;

   procedure Set_Character_Transmission_Time
     (The_Driver : in out Character_Packet_Driver;
      The_Transmission_Time : Time);
   function Character_Transmission_Time
     (The_Driver : Character_Packet_Driver)
     return Time;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Drv : in out Character_Packet_Driver;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   function Clone
     (Drv : Character_Packet_Driver)
     return  Driver_Ref;

private

   type Driver is abstract tagged null record;

   type Packet_Driver is new Driver with record
      Packet_Server : Scheduling_Servers.Scheduling_Server_Ref;
      Packet_Send_Operation,
        Packet_receive_Operation: Operations.Simple_Operation_Ref;
   end record;

   type Character_Packet_Driver is new Packet_Driver with record
      Character_Send_Operation,
        Character_receive_Operation: Operations.Simple_Operation_Ref;
      Character_Server : Scheduling_Servers.Scheduling_Server_Ref;
      Character_Transmission_Time : Time:=Large_Time;
   end record;

end Mast.Drivers;
