-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with MAST.IO;
package body MAST.Graphs.Event_Handlers is

   use type MAST.Operations.Operation_Ref;
   use type MAST.Scheduling_Servers.Scheduling_Server_Ref;
   use type MAST.Events.Event_Ref;


   ----------------------
   -- Incomplete_Clone --
   ----------------------

   function Incomplete_Clone
     (The_List : Link_Lists.List)
     return Link_Lists.List
   is
      The_Copy : Link_Lists.List;
      Iterator : Link_Lists.Iteration_Object;
      Lnk_Ref  : Link_Ref;
   begin
      Link_Lists.Rewind(The_List,Iterator);
      for I in 1..Link_Lists.Size(The_List) loop
         Link_Lists.Get_Next_Item(Lnk_Ref,The_List,Iterator);
         Link_Lists.Add(Lnk_Ref,The_Copy);
      end loop;
      return The_Copy;
   end Incomplete_Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : Activity)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new Activity'(Ev_Hdlr);
      return Ev_Ref;
   end Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : System_Timed_Activity)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new System_Timed_Activity'(Ev_Hdlr);
      return Ev_Ref;
   end Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : Concentrator)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new Concentrator'(Ev_Hdlr);
      Concentrator(Ev_Ref.all).Input_Links:=
        Incomplete_Clone(Ev_Hdlr.Input_Links);
      return Ev_Ref;
   end Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : Barrier)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new Barrier'(Ev_Hdlr);
      Barrier(Ev_Ref.all).Input_Links:=Incomplete_Clone(Ev_Hdlr.Input_Links);
      return Ev_Ref;
   end Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : Delivery_Server)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new Delivery_Server'(Ev_Hdlr);
      Delivery_Server(Ev_Ref.all).Output_Links:=
        Incomplete_Clone(Ev_Hdlr.Output_Links);
      return Ev_Ref;
   end Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : Query_Server)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new Query_Server'(Ev_Hdlr);
      Query_Server(Ev_Ref.all).Output_Links:=
        Incomplete_Clone(Ev_Hdlr.Output_Links);
      return Ev_Ref;
   end Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : Multicast)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new Multicast'(Ev_Hdlr);
      Multicast(Ev_Ref.all).Output_Links:=
        Incomplete_Clone(Ev_Hdlr.Output_Links);
      return Ev_Ref;
   end Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : Rate_Divisor)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new Rate_Divisor'(Ev_Hdlr);
      return Ev_Ref;
   end Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : Delay_Event_Handler)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new Delay_Event_Handler'(Ev_Hdlr);
      return Ev_Ref;
   end Clone;

   -----------
   -- Clone --
   -----------

   function Clone
     (Ev_Hdlr : Offset_Event_Handler)
     return Event_Handler_Ref
   is
      Ev_Ref : Event_Handler_Ref;
   begin
      Ev_Ref:=new Offset_Event_Handler'(Ev_Hdlr);
      return Ev_Ref;
   end Clone;

   -----------------------
   -- Set_Activity_Server --
   -----------------------

   procedure Set_Activity_Server
     (The_Event_Handler : in out Activity;
      The_Activity_Server : Scheduling_Servers.Scheduling_Server_Ref)
   is
   begin
      The_Event_Handler.Activity_Server:=The_Activity_Server;
   end Set_Activity_Server;

   -------------------
   -- Activity_Server --
   -------------------

   function Activity_Server
     (The_Event_Handler : Activity)
      return Scheduling_Servers.Scheduling_Server_Ref
   is
   begin
      return The_Event_Handler.Activity_Server;
   end Activity_Server;

   -------------------
   -- Add_Input_Link --
   -------------------

   procedure Add_Input_Link
     (The_Event_Handler : in out Input_Event_Handler;
      The_Link : Link_Ref)
   is
   begin
      Link_Lists.Add(The_Link,The_Event_Handler.Input_Links);
   end Add_Input_Link;

   --------------------
   -- Add_Output_Link --
   --------------------

   procedure Add_Output_Link
     (The_Event_Handler : in out Output_Event_Handler;
      The_Link : Link_Ref)
   is
   begin
      Link_Lists.Add(The_Link,The_Event_Handler.Output_Links);
   end Add_Output_Link;

   ------------------------
   -- Delay_Max_Interval --
   ------------------------

   function Delay_Max_Interval
     (The_Event_Handler : Delay_Event_Handler)
      return Time
   is
   begin
      return The_Event_Handler.Delay_Max_Interval;
   end Delay_Max_Interval;

   ------------------------
   -- Delay_Min_Interval --
   ------------------------

   function Delay_Min_Interval
     (The_Event_Handler : Delay_Event_Handler)
      return Time
   is
   begin
      return The_Event_Handler.Delay_Min_Interval;
   end Delay_Min_Interval;

   --------------------
   -- Referenced_Event --
   --------------------

   function Referenced_Event
     (The_Event_Handler : Offset_Event_Handler)
      return MAST.Events.Event_Ref
   is
   begin
      return The_Event_Handler.Referenced_Event;
   end Referenced_Event;

   ------------------------
   -- Get_Next_Input_Link --
   ------------------------

   procedure Get_Next_Input_Link
     (The_Event_Handler : in Input_Event_Handler;
      The_Link : out Link_Ref;
      Iterator : in out Iteration_Object)
   is
   begin
      Link_Lists.Get_Next_Item
        (The_Link,The_Event_Handler.Input_Links,
         Link_Lists.Index(Iterator));
   end Get_Next_Input_Link;

   -------------------------
   -- Get_Next_Output_Link --
   -------------------------

   procedure Get_Next_Output_Link
     (The_Event_Handler : in Output_Event_Handler;
      The_Link : out Link_Ref;
      Iterator : in out Iteration_Object)
   is
   begin
      Link_Lists.Get_Next_Item
        (The_Link,The_Event_Handler.Output_Links,
         Link_Lists.Index(Iterator));
   end Get_Next_Output_Link;

   ---------------
   -- Input_Link --
   ---------------

   function Input_Link
     (The_Event_Handler : Simple_Event_Handler)
      return Link_Ref
   is
   begin
      return The_Event_Handler.Input_Link;
   end Input_Link;

   ---------------
   -- Input_Link --
   ---------------

   function Input_Link
     (The_Event_Handler : Output_Event_Handler)
      return Link_Ref
   is
   begin
      return The_Event_Handler.Input_Link;
   end Input_Link;

   -----------------
   -- Activity_Operation --
   -----------------

   function Activity_Operation
     (The_Event_Handler : Activity)
      return MAST.Operations.Operation_Ref
   is
   begin
      return The_Event_Handler.Activity_Operation;
   end Activity_Operation;

   -----------------------
   -- Num_Of_Input_Links --
   -----------------------

   function Num_Of_Input_Links
     (The_Event_Handler : Input_Event_Handler)
      return Natural
   is
   begin
      return Link_Lists.Size(The_Event_Handler.Input_Links);
   end Num_Of_Input_Links;

   ------------------------
   -- Num_Of_Output_Links --
   ------------------------

   function Num_Of_Output_Links
     (The_Event_Handler : Output_Event_Handler)
      return Natural
   is
   begin
      return Link_Lists.Size(The_Event_Handler.Output_Links);
   end Num_Of_Output_Links;

   ----------------
   -- Output_Link --
   ----------------

   function Output_Link
     (The_Event_Handler : Simple_Event_Handler)
      return Link_Ref
   is
   begin
      return The_Event_Handler.Output_Link;
   end Output_Link;

   ----------------
   -- Output_Link --
   ----------------

   function Output_Link
     (The_Event_Handler : Input_Event_Handler)
      return Link_Ref
   is
   begin
      return The_Event_Handler.Output_Link;
   end Output_Link;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Simple_Event_Handler;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 19;
   begin
      if Res.Input_Link/=null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Input_Event",
            IO.Name_Image(MAST.Graphs.Name(Res.Input_Link)),
            Indentation+1,Names_Length-1);
      end if;
      if Res.Output_Link/= null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Output_Event",
           IO.Name_Image(MAST.Graphs.Name(Res.Output_Link)),
            Indentation+1,Names_Length-1);
      end if;
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Activity;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 19;
   begin
      MAST.IO.Print_Arg(File,"(Type","Activity",
                        Indentation,Names_Length);
      Print(File,Simple_Event_Handler(Res),Indentation);
      if Res.Activity_Operation /= null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Activity_Operation",
            IO.Name_Image(MAST.Operations.Name
                          (Res.Activity_Operation)),
            Indentation+1,Names_Length-1);
      end if;
      if Res.Activity_Server /= null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Activity_Server",
            IO.Name_Image(MAST.Scheduling_Servers.Name
                          (Res.Activity_Server)),
            Indentation+1,Names_Length-1);
      end if;
      Ada.Text_IO.Put(File,")");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out System_Timed_Activity;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 19;
   begin
      MAST.IO.Print_Arg(File,"(Type","System_Timed_Activity",
                        Indentation,Names_Length);
      Print(File,Simple_Event_Handler(Res),Indentation);
      if Res.Activity_Operation /= null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Activity_Operation",
            IO.Name_Image(MAST.Operations.Name
                          (Res.Activity_Operation)),
            Indentation+1,Names_Length-1);
      end if;
      if Res.Activity_Server /= null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Activity_Server",
            IO.Name_Image(MAST.Scheduling_Servers.Name
                          (Res.Activity_Server)),
            Indentation+1,Names_Length-1);
      end if;
      Ada.Text_IO.Put(File,")");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Input_Event_Handler;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 17;
      Res_Ref : Link_Ref;
      Iterator : Link_Lists.Index;
   begin
      if Res.Output_Link/=null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Output_Event",
            IO.Name_Image(MAST.Graphs.Name(Res.Output_Link)),
            Indentation+1,Names_Length-1);
      end if;
      if Link_Lists.Size(Res.Input_Links)>0 then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg(File,"Input_Events_List","",
                           Indentation+1,Names_Length-1);
         Link_Lists.Rewind(Res.Input_Links,Iterator);
         for I in 1..Link_Lists.Size(Res.Input_Links) loop
            if I = 1 then
               MAST.IO.Print_Separator(File,MAST.IO.Nothing);
               MAST.IO.Print_List_Item
                 (File,MAST.IO.Left_Paren,Indentation+4);
            end if;
            Link_Lists.Get_Next_Item(Res_Ref,Res.Input_Links,Iterator);
            MAST.IO.Print_List_Item
              (File, IO.Name_Image
               (MAST.Graphs.Name(Res_Ref)),Indentation+6);
            if I = Link_Lists.Size(Res.Input_Links) then
               Ada.Text_IO.Put(File,")");
            else
               MAST.IO.Print_Separator(File);
            end if;
         end loop;
      end if;
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Concentrator;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 17;
   begin
      MAST.IO.Print_Arg(File,"(Type","Concentrator",
                        Indentation,Names_Length);
      Print(File,Input_Event_Handler(Res),Indentation);
      Ada.Text_IO.Put(File,")");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Barrier;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 17;
   begin
      MAST.IO.Print_Arg(File,"(Type","Barrier",
                        Indentation,Names_Length);
      Print(File,Input_Event_Handler(Res),Indentation);
      Ada.Text_IO.Put(File,")");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Output_Event_Handler;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 18;
      Res_Ref : Link_Ref;
      Iterator : Link_Lists.Index;
   begin
      if Res.Input_Link/=null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Input_Event",
            IO.Name_Image(MAST.Graphs.Name(Res.Input_Link)),
            Indentation+1,Names_Length-1);
      end if;
      if Link_Lists.Size(Res.Output_Links)>0 then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg(File,"Output_Events_List","",
                           Indentation+1,Names_Length-1);
         Link_Lists.Rewind(Res.Output_Links,Iterator);
         for I in 1..Link_Lists.Size(Res.Output_Links) loop
            if I = 1 then
               MAST.IO.Print_Separator(File,MAST.IO.Nothing);
               MAST.IO.Print_List_Item
                 (File,MAST.IO.Left_Paren,Indentation+4);
            end if;
            Link_Lists.Get_Next_Item(Res_Ref,Res.Output_Links,Iterator);
            MAST.IO.Print_List_Item
              (File,IO.Name_Image
               (MAST.Graphs.Name(Res_Ref)),Indentation+6);
            if I = Link_Lists.Size(Res.Output_Links) then
               Ada.Text_IO.Put(File,")");
            else
               MAST.IO.Print_Separator(File);
            end if;
         end loop;
      end if;
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Delivery_Server;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 18;
   begin
      MAST.IO.Print_Arg(File,"(Type","Delivery_Server",
                        Indentation,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg(File,"Delivery_Policy",
                        Delivery_Policy'Image(Res.Policy),
                        Indentation+1,Names_Length-1);
      Print(File,Output_Event_Handler(Res),Indentation);
      Ada.Text_IO.Put(File,")");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Query_Server;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 18;
   begin
      MAST.IO.Print_Arg(File,"(Type","Query_Server",
                        Indentation,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg(File,"Request_Policy",
                        Request_Policy'Image(Res.Policy),
                        Indentation+1,Names_Length-1);
      Print(File,Output_Event_Handler(Res),Indentation);
      Ada.Text_IO.Put(File,")");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Multicast;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 18;
   begin
      MAST.IO.Print_Arg(File,"(Type","Multicast",
                        Indentation,Names_Length);
      Print(File,Output_Event_Handler(Res),Indentation);
      Ada.Text_IO.Put(File,")");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Rate_Divisor;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 19;
   begin
      MAST.IO.Print_Arg(File,"(Type","Rate_Divisor",
                        Indentation,Names_Length);
      Print(File,Simple_Event_Handler(Res),Indentation);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg(File,"Rate_Factor",
                        Positive'Image(Res.Rate_Factor),
                        Indentation+1,Names_Length-1);
      Ada.Text_IO.Put(File,")");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Delay_Event_Handler;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 19;
   begin
      MAST.IO.Print_Arg(File,"(Type","Delay",
                        Indentation,Names_Length);
      Print(File,Simple_Event_Handler(Res),Indentation);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg(File,"Delay_Max_Interval",
                        IO.Time_Image(Res.Delay_Max_Interval),
                        Indentation+1,Names_Length-1);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg(File,"Delay_Min_Interval",
                        IO.Time_Image(Res.Delay_Min_Interval),
                        Indentation+1,Names_Length-1);
      Ada.Text_IO.Put(File,")");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Offset_Event_Handler;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant Positive := 19;
   begin
      MAST.IO.Print_Arg(File,"(Type","Offset",
                        Indentation,Names_Length);
      Print(File,Simple_Event_Handler(Res),Indentation);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg(File,"Delay_Max_Interval",
                        IO.Time_Image(Res.Delay_Max_Interval),
                        Indentation+1,Names_Length-1);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg(File,"Delay_Min_Interval",
                        IO.Time_Image(Res.Delay_Min_Interval),
                        Indentation+1,Names_Length-1);
      if Res.Referenced_Event/=null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Referenced_Event",
            IO.Name_Image(MAST.Events.Name
                          (Res.Referenced_Event)),
            Indentation+1,Names_Length-1);
      end if;
      Ada.Text_IO.Put(File,")");
   end Print;

   -----------------
   -- Rate_Factor --
   -----------------

   function Rate_Factor
     (The_Event_Handler : Rate_Divisor)
      return Positive
   is
   begin
      return The_Event_Handler.Rate_Factor;
   end Rate_Factor;

   -----------------------
   -- Rewind_Input_Links --
   -----------------------

   procedure Rewind_Input_Links
     (The_Event_Handler : in Input_Event_Handler;
      Iterator : out Iteration_Object)
   is
   begin
      Link_Lists.Rewind
        (The_Event_Handler.Input_Links,Link_Lists.Index(Iterator));
   end Rewind_Input_Links;

   ------------------------
   -- Rewind_Output_Links --
   ------------------------

   procedure Rewind_Output_Links
     (The_Event_Handler : in Output_Event_Handler;
      Iterator : out Iteration_Object)
   is
   begin
      Link_Lists.Rewind
        (The_Event_Handler.Output_Links,Link_Lists.Index(Iterator));
   end Rewind_Output_Links;

   ----------------------------
   -- Set_Delay_Max_Interval --
   ----------------------------

   procedure Set_Delay_Max_Interval
     (The_Event_Handler : in out Delay_Event_Handler;
      The_Delay_Max_Interval : Time)
   is
   begin
      The_Event_Handler.Delay_Max_Interval:=The_Delay_Max_Interval;
   end Set_Delay_Max_Interval;

   ----------------------------
   -- Set_Delay_Min_Interval --
   ----------------------------

   procedure Set_Delay_Min_Interval
     (The_Event_Handler : in out Delay_Event_Handler;
      The_Delay_Min_Interval : Time)
   is
   begin
      The_Event_Handler.Delay_Min_Interval:=The_Delay_Min_Interval;
   end Set_Delay_Min_Interval;

   ------------------------
   -- Set_Referenced_Event --
   ------------------------

   procedure Set_Referenced_Event
     (The_Event_Handler : in out Offset_Event_Handler;
      The_Referenced_Event : MAST.Events.Event_Ref)
   is
   begin
      The_Event_Handler.Referenced_Event:=The_Referenced_Event;
   end Set_Referenced_Event;

   -------------------
   -- Set_Input_Link --
   -------------------

   procedure Set_Input_Link
     (The_Event_Handler : in out Simple_Event_Handler;
      The_Input_Link : Link_Ref)
   is
   begin
      The_Event_Handler.Input_Link:=The_Input_Link;
   end Set_Input_Link;

   -------------------
   -- Set_Input_Link --
   -------------------

   procedure Set_Input_Link
     (The_Event_Handler : in out Output_Event_Handler;
      The_Input_Link : Link_Ref)
   is
   begin
      The_Event_Handler.Input_Link:=The_Input_Link;
   end Set_Input_Link;

   ---------------------
   -- Set_Activity_Operation --
   ---------------------

   procedure Set_Activity_Operation
     (The_Event_Handler : in out Activity;
      The_Activity_Operation : MAST.Operations.Operation_Ref)
   is
   begin
      The_Event_Handler.Activity_Operation:=The_Activity_Operation;
   end Set_Activity_Operation;

   --------------------
   -- Set_Output_Link --
   --------------------

   procedure Set_Output_Link
     (The_Event_Handler : in out Simple_Event_Handler;
      The_Output_Link : Link_Ref)
   is
   begin
      The_Event_Handler.Output_Link:=The_Output_Link;
   end Set_Output_Link;

   --------------------
   -- Set_Output_Link --
   --------------------

   procedure Set_Output_Link
     (The_Event_Handler : in out Input_Event_Handler;
      The_Output_Link : Link_Ref)
   is
   begin
      The_Event_Handler.Output_Link:=The_Output_Link;
   end Set_Output_Link;

   ---------------------
   -- Set_Rate_Factor --
   ---------------------

   procedure Set_Rate_Factor
     (The_Event_Handler : in out Rate_Divisor;
      The_Rate_Factor : Positive)
   is
   begin
      The_Event_Handler.Rate_Factor:=The_Rate_Factor;
   end Set_Rate_Factor;

   ---------------------
   -- Set_Policy --
   ---------------------

   procedure Set_Policy
     (The_Event_Handler : in out Delivery_Server;
      The_Policy : Mast.Delivery_Policy)
   is
   begin
      The_Event_Handler.Policy:=The_Policy;
   end Set_Policy;

   ---------------------
   -- Policy --
   ---------------------

   function Policy
     (The_Event_Handler : Delivery_Server)
      return Mast.Delivery_Policy
   is
   begin
      return The_Event_Handler.Policy;
   end Policy;

   ---------------------
   -- Set_Policy --
   ---------------------

   procedure Set_Policy
     (The_Event_Handler : in out Query_Server;
      The_Policy : Mast.Request_Policy)
   is
   begin
      The_Event_Handler.Policy:=The_Policy;
   end Set_Policy;

   ---------------------
   -- Policy --
   ---------------------

   function Policy
     (The_Event_Handler : Query_Server)
      return Mast.Request_Policy
   is
   begin
      return The_Event_Handler.Policy;
   end Policy;

end MAST.Graphs.Event_Handlers;
