-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with MAST.IO;
package body Mast.Processing_Resources.FP is

   use type MAST.Drivers.Driver_Ref;
   use type Mast.Timers.System_Timer_Ref;

   ------------
   -- Clone  --
   ------------

   function Clone
       (Res : Fixed_Priority_Processor)
       return  Processing_Resource_Ref
   is
      Res_Ref : Processing_Resource_Ref;
   begin
      Res_Ref:=new Fixed_Priority_Processor'(Res);
      -- Clone the system timer
      if Res.The_System_Timer/=null then
         Fixed_Priority_Processor(Res_Ref.all).The_System_Timer:=
           Timers.Clone(Res.The_System_Timer.all);
      end if;
      return Res_Ref;
   end Clone;

   ------------
   -- Clone  --
   ------------
   function Clone
       (Res : Fixed_Priority_Network)
       return  Processing_Resource_Ref
   is
      Res_Ref : Processing_Resource_Ref;
      Null_List : Drivers.Lists.List;
   begin
      Res_Ref:=new Fixed_Priority_Network'(Res);
      Fixed_Priority_Network(Res_Ref.all).List_Of_Drivers:=Null_List;
      Fixed_Priority_Network(Res_Ref.all).List_Of_Drivers:=
        Drivers.Clone(Res.List_Of_Drivers);
      return Res_Ref;
   end Clone;

   ------------------------
   -- Add_Driver         --
   ------------------------

   procedure Add_Driver
     (Net : in out Network;
      The_Driver : MAST.Drivers.Driver_Ref)
   is
   begin
      Drivers.Lists.Add(The_Driver,Net.List_Of_Drivers);
   end Add_Driver;

   ------------------------
   -- Num_of_Drivers     --
   ------------------------

   function Num_Of_Drivers
     (Net : Network)
      return Natural
   is
   begin
      return Drivers.Lists.Size(Net.List_Of_Drivers);
   end Num_Of_Drivers;

   ------------------------
   -- Rewind_Drivers     --
   ------------------------

   procedure Rewind_Drivers
     (Net : in Network;
      Iterator : out Driver_Iteration_Object)
   is
   begin
      Drivers.Lists.Rewind
        (Net.List_Of_Drivers,
         Drivers.Lists.Iteration_Object(Iterator));
   end Rewind_Drivers;

   ------------------------
   -- Get_Next_Driver    --
   ------------------------

   procedure Get_Next_Driver
     (Net : in Network;
      The_Driver : out MAST.Drivers.Driver_Ref;
      Iterator : in out Driver_Iteration_Object)
   is
   begin
      Drivers.Lists.Get_Next_Item
        (The_Driver,Net.List_Of_Drivers,
         Drivers.Lists.Iteration_Object(Iterator));
   end Get_Next_Driver;

   ------------------------
   -- Avg_Context_Switch --
   ------------------------

   function Avg_Context_Switch
     (CPU : Fixed_Priority_Processor)
      return Time
   is
   begin
      return CPU.Avg_Context_Switch;
   end Avg_Context_Switch;

   ------------------------
   -- Avg_ISR_Switch --
   ------------------------

   function Avg_ISR_Switch
     (CPU : Fixed_Priority_Processor)
      return Time
   is
   begin
      return CPU.Avg_ISR_Switch;
   end Avg_ISR_Switch;

   -------------------------
   -- Best_Context_Switch --
   -------------------------

   function Best_Context_Switch
     (CPU : Fixed_Priority_Processor)
      return Time
   is
   begin
      return CPU.Best_Context_Switch;
   end Best_Context_Switch;

   -------------------------
   -- Best_ISR_Switch --
   -------------------------

   function Best_ISR_Switch
     (CPU : Fixed_Priority_Processor)
      return Time
   is
   begin
      return CPU.Best_ISR_Switch;
   end Best_ISR_Switch;

   ------------------
   -- Max_Blocking --
   ------------------

   function Max_Blocking
     (Net  : Fixed_Priority_Network)
      return Time
   is
   begin
      return Net.Max_Blocking;
   end Max_Blocking;

   ---------------------------------
   -- Max_Packet_Transmission_Time --
   ---------------------------------

   function Max_Packet_Transmission_Time
     (Net  : Fixed_Priority_Network)
      return Time
   is
   begin
      return Net.Max_Packet_Transmission_Time;
   end Max_Packet_Transmission_Time;

   ---------------------------------
   -- Min_Packet_Transmission_Time --
   ---------------------------------

   function Min_Packet_Transmission_Time
     (Net  : Fixed_Priority_Network)
      return Time
   is
   begin
      return Net.Min_Packet_Transmission_Time;
   end Min_Packet_Transmission_Time;

   ---------------------------------
   -- Packet_Avg_Overhead --
   ---------------------------------

   function Packet_Avg_Overhead
     (Net  : Fixed_Priority_Network)
      return Time
   is
   begin
      return Net.Packet_Avg_Overhead;
   end Packet_Avg_Overhead;

   ----------------------------------
   -- Packet_Best_Overhead --
   ----------------------------------

   function Packet_Best_Overhead
     (Net  : Fixed_Priority_Network)
      return Time
   is
   begin
      return Net.Packet_Best_Overhead;
   end Packet_Best_Overhead;

   -----------------------------------
   -- Packet_Worst_Overhead --
   -----------------------------------

   function Packet_Worst_Overhead
     (Net  : Fixed_Priority_Network)
      return Time
   is
   begin
      return Net.Packet_Worst_Overhead;
   end Packet_Worst_Overhead;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Fixed_Priority_Processor;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Name_Length : constant := 22;
   begin
      Print(File,Processing_Resource(Res),Indentation);
      MAST.IO.Print_Arg
        (File,"Type",
         "Fixed_Priority_Processor",Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Name",
         IO.Name_Image(Res.Name),Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Worst_Context_Switch",
         IO.Time_Image(Res.Worst_Context_Switch),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Max_Priority",
         Priority'Image(Res.Max_Priority),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Min_Priority",
         Priority'Image(Res.Min_Priority),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Max_Interrupt_Priority",
         Priority'Image(Res.Max_Interrupt_Priority),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Min_Interrupt_Priority",
         Priority'Image(Res.Min_Interrupt_Priority),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Avg_Context_Switch",
         IO.Time_Image(Res.Avg_Context_Switch),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Best_Context_Switch",
         IO.Time_Image(Res.Best_Context_Switch),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Worst_ISR_Switch",
         IO.Time_Image(Res.Worst_ISR_Switch),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Avg_ISR_Switch",
         IO.Time_Image(Res.Avg_ISR_Switch),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Best_ISR_Switch",
         IO.Time_Image(Res.Best_ISR_Switch),
         Indentation+3,Name_Length);
      if Res.The_System_Timer/=null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"System_Timer","",
            Indentation+3,Name_Length);
         Timers.Print(File,Res.The_System_Timer.all,Indentation+6);
      end if;
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Speed_Factor",
         IO.Speed_Image(Res.Speed_Factor),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File,",",Finalize);
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Fixed_Priority_Network;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Name_Length : constant := 29;
   begin
      Print(File,Processing_Resource(Res),Indentation);
      MAST.IO.Print_Arg
        (File,"Type",
         "Fixed_Priority_Network",Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Name",
         IO.Name_Image(Res.Name),Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Max_Priority",
         Priority'Image(Res.Max_Priority),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Min_Priority",
         Priority'Image(Res.Min_Priority),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Transmission",
         Transmission_Kind'Image(Res.Transmission_Mode),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Packet_Worst_Overhead",
         IO.Time_Image(Res.Packet_Worst_Overhead),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Packet_Avg_Overhead",
         IO.Time_Image(Res.Packet_Avg_Overhead),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Packet_Best_Overhead",
         IO.Time_Image(Res.Packet_Best_Overhead),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Max_Blocking",
         IO.Time_Image(Res.Max_Blocking),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Max_Packet_Transmission_Time",
         IO.Time_Image(Res.Max_Packet_Transmission_Time),
         Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Min_Packet_Transmission_Time",
         IO.Time_Image(Res.Min_Packet_Transmission_Time),
         Indentation+3,Name_Length);
      if Drivers.Lists.Size(Res.List_Of_Drivers) >0 then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"List_of_Drivers","",
            Indentation+3,Name_Length);
         Ada.Text_IO.New_Line(File);
         Drivers.Print(File,Res.List_Of_Drivers,Indentation+6);
      end if;
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Speed_Factor",
         IO.Speed_Image(Res.Speed_Factor),Indentation+3,Name_Length);
      MAST.IO.Print_Separator(File,",",Finalize);
   end Print;

   ----------------------------
   -- Set_Avg_Context_Switch --
   ----------------------------

   procedure Set_Avg_Context_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Avg_Context_Switch : Time)
   is
   begin
      CPU.Avg_Context_Switch:=The_Avg_Context_Switch;
   end Set_Avg_Context_Switch;

   ----------------------------
   -- Set_Avg_ISR_Switch --
   ----------------------------

   procedure Set_Avg_ISR_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Avg_ISR_Switch : Time)
   is
   begin
      CPU.Avg_ISR_Switch:=The_Avg_ISR_Switch;
   end Set_Avg_ISR_Switch;

   -----------------------------
   -- Set_Best_Context_Switch --
   -----------------------------

   procedure Set_Best_Context_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Best_Context_Switch : Time)
   is
   begin
      CPU.Best_Context_Switch:=The_Best_Context_Switch;
   end Set_Best_Context_Switch;

   -----------------------------
   -- Set_Best_ISR_Switch --
   -----------------------------

   procedure Set_Best_ISR_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Best_ISR_Switch : Time)
   is
   begin
      CPU.Best_ISR_Switch:=The_Best_ISR_Switch;
   end Set_Best_ISR_Switch;

   ------------------------------
   -- Set_Worst_Context_Switch --
   ------------------------------

   procedure Set_Worst_Context_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Worst_Context_Switch : Time)
   is
   begin
      CPU.Worst_Context_Switch:=The_Worst_Context_Switch;
   end Set_Worst_Context_Switch;

   ------------------------------
   -- Set_Worst_ISR_Switch --
   ------------------------------

   procedure Set_Worst_ISR_Switch
     (CPU : in out Fixed_Priority_Processor;
      The_Worst_ISR_Switch : Time)
   is
   begin
      CPU.Worst_ISR_Switch:=The_Worst_ISR_Switch;
   end Set_Worst_ISR_Switch;

   ----------------------
   -- Set_Max_Blocking --
   ----------------------

   procedure Set_Max_Blocking
     (Net  : in out Fixed_Priority_Network;
      The_Max_Blocking : Time)
   is
   begin
      Net.Max_Blocking:=The_Max_Blocking;
   end Set_Max_Blocking;

   -------------------------------------
   -- Set_Max_Packet_Transmission_Time --
   ------------------------------------

   procedure Set_Max_Packet_Transmission_Time
     (Net  : in out Fixed_Priority_Network;
      The_Max_Transmission_Time : Time)
   is
   begin
      Net.Max_Packet_Transmission_Time:=The_Max_Transmission_Time;
   end Set_Max_Packet_Transmission_Time;

   -------------------------------------
   -- Set_Min_Packet_Transmission_Time --
   ------------------------------------

   procedure Set_Min_Packet_Transmission_Time
     (Net  : in out Fixed_Priority_Network;
      The_Min_Transmission_Time : Time)
   is
   begin
      Net.Min_Packet_Transmission_Time:=The_Min_Transmission_Time;
   end Set_Min_Packet_Transmission_Time;

   -------------------------------------
   -- Set_Packet_Avg_Overhead --
   -------------------------------------

   procedure Set_Packet_Avg_Overhead
     (Net : in out Fixed_Priority_Network;
      The_Avg_Overhead : Time)
   is
   begin
      Net.Packet_Avg_Overhead:=The_Avg_Overhead;
   end Set_Packet_Avg_Overhead;

   --------------------------------------
   -- Set_Packet_Best_Overhead --
   --------------------------------------

   procedure Set_Packet_Best_Overhead
     (Net  : in out Fixed_Priority_Network;
      The_Best_Overhead : Time)
   is
   begin
      Net.Packet_Best_Overhead:=The_Best_Overhead;
   end Set_Packet_Best_Overhead;

   ---------------------------------------
   -- Set_Packet_Worst_Overhead --
   ---------------------------------------

   procedure Set_Packet_Worst_Overhead
     (Net  : in out Fixed_Priority_Network;
      The_Worst_Overhead : Time)
   is
   begin
      Net.Packet_Worst_Overhead:=The_Worst_Overhead;
   end Set_Packet_Worst_Overhead;

   ---------------------------
   -- Set_Transmission_Mode --
   ---------------------------

   procedure Set_Transmission_Mode
     (Net  : in out Fixed_Priority_Network;
      The_Transmission_Mode : Transmission_Kind)
   is
   begin
      Net.Transmission_Mode:=The_Transmission_Mode;
   end Set_Transmission_Mode;

   ---------------------------
   -- Set_Transmission_Mode --
   ---------------------------

   function Transmission_Mode
     (Net  : Fixed_Priority_Network) return Transmission_Kind
   is
   begin
      return Net.Transmission_Mode;
   end Transmission_Mode;

   --------------------------
   -- Worst_Context_Switch --
   --------------------------

   function Worst_Context_Switch
     (CPU : Fixed_Priority_Processor)
      return Time
   is
   begin
      return CPU.Worst_Context_Switch;
   end Worst_Context_Switch;

   --------------------------
   -- Worst_ISR_Switch --
   --------------------------

   function Worst_ISR_Switch
     (CPU : Fixed_Priority_Processor)
      return Time
   is
   begin
      return CPU.Worst_ISR_Switch;
   end Worst_ISR_Switch;

   ------------------------------------
   -- Set_Max_Interrupt_Priority --
   ------------------------------------

   procedure Set_Max_Interrupt_Priority
     (Res : in out Fixed_Priority_Processor;
      The_Priority : Priority)
   is
   begin
      Res.Max_Interrupt_Priority:=The_Priority;
   end Set_Max_Interrupt_Priority;

   ------------------------------------
   -- max_interrupt_priority --
   ------------------------------------

   function Max_Interrupt_Priority
     (Res : Fixed_Priority_Processor)
     return Priority
   is
   begin
      return Res.Max_Interrupt_Priority;
   end Max_Interrupt_Priority;

   ------------------------------------
   -- Set_Min_Interrupt_Priority --
   ------------------------------------

   procedure Set_Min_Interrupt_Priority
     (Res : in out Fixed_Priority_Processor;
      The_Priority : Priority)
   is
   begin
      Res.Min_Interrupt_Priority:=The_Priority;
   end Set_Min_Interrupt_Priority;

   ------------------------------------
   -- min_interrupt_priority --
   ------------------------------------

   function Min_Interrupt_Priority
     (Res : Fixed_Priority_Processor)
     return Priority
   is
   begin
      return Res.Min_Interrupt_Priority;
   end Min_Interrupt_Priority;

   ------------------------------------
   -- Set_System_Timer --
   ------------------------------------

   procedure Set_System_Timer
     (CPU : in out Fixed_Priority_Processor;
      The_System_Timer : Timers.System_Timer_Ref)
   is
   begin
      CPU.The_System_Timer:=The_System_Timer;
   end Set_System_Timer;

   ------------------------------------
   -- The_System_Timer --
   ------------------------------------

   function The_System_Timer
     (CPU : Fixed_Priority_Processor)
     return Timers.System_Timer_Ref
   is
   begin
      return CPU.The_System_Timer;
   end The_System_Timer;

   ------------------------------------
   -- Max_Any_Priority --
   ------------------------------------

   function Max_Any_Priority (Res : Fixed_Priority_Processor)
     return Priority
   is
   begin
      return Res.Max_Interrupt_Priority;
   end Max_Any_Priority;

end Mast.Processing_Resources.FP;
