-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

package body Mast.Processing_Resources is

   -----------
   -- Clone --
   -----------

   function Clone
     (The_List : in Lists.List)
     return Lists.List
   is
      The_Copy : Lists.List;
      Iterator : Lists.Iteration_Object;
      Res_Ref, Res_Ref_Copy : Processing_Resource_Ref;
   begin
      Lists.Rewind(The_List,Iterator);
      for I in 1..Lists.Size(The_List) loop
         Lists.Get_Next_Item(Res_Ref,The_List,Iterator);
         Res_Ref_Copy:=Clone(Res_Ref.all);
         Lists.Add(Res_Ref_Copy,The_Copy);
      end loop;
      return The_Copy;
   end Clone;


   ----------
   -- Init --
   ----------

   procedure Init
     (Res : in out Processing_Resource;
      Name : Var_Strings.Var_String)
   is
   begin
      Res.Name:=Name;
   end Init;

   ----------
   -- Name --
   ----------

   function Name
     (Res : Processing_Resource)
      return Var_Strings.Var_String
   is
   begin
      return Res.Name;
   end Name;

   ----------
   -- Name --
   ----------

   function Name
     (Res_Ref : Processing_Resource_Ref)
      return Var_Strings.Var_String
   is
   begin
      return Res_Ref.Name;
   end Name;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Processing_Resource;
      Indentation : Positive;
      Finalize    : Boolean:=False) is
   begin
      Ada.Text_IO.Set_Col(File,Ada.Text_IO.Count(Indentation));
      Ada.Text_IO.Put(File,"Processing_Resource (");
   end Print;

   --------------------------------
   -- Print                      --
   --------------------------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      The_List : in out Lists.List;
      Indentation : Positive)
   is
      Res_Ref : Processing_Resource_Ref;
      Iterator : LIsts.Index;
   begin
      Lists.Rewind(The_List,Iterator);
      for I in 1..Lists.Size(The_List) loop
         Lists.Get_Next_Item(Res_Ref,The_List,Iterator);
         Print(File,Res_Ref.all,Indentation,True);
         Ada.Text_IO.New_Line(File);
      end loop;
   end Print;

   ------------------------------------
   -- Set_Speed_Factor --
   ------------------------------------

   procedure Set_Speed_Factor (Res : in out Processing_Resource;
                               The_Speed_Factor : Processor_Speed)
   is
   begin
      Res.Speed_Factor:=The_Speed_Factor;
   end Set_Speed_Factor;

   ------------------------------------
   -- Speed_Factor --
   ------------------------------------

   function Speed_Factor (Res : Processing_Resource)
                          return Processor_Speed
   is
   begin
      return Res.Speed_Factor;
   end Speed_Factor;

   ------------------------------------
   -- Set_Max_Priority --
   ------------------------------------

   procedure Set_Max_Priority
     (Res : in out Processing_Resource;
      The_Priority : Priority)
   is
   begin
      Res.Max_Priority:=The_Priority;
   end Set_Max_Priority;

   ------------------------------------
   -- max_priority --
   ------------------------------------

   function Max_Priority
     (Res : Processing_Resource)
     return Priority
   is
   begin
      return Res.Max_Priority;
   end Max_Priority;

   ------------------------------------
   -- Set_Min_Priority --
   ------------------------------------

   procedure Set_Min_Priority
     (Res : in out Processing_Resource;
      The_Priority : Priority)
   is
   begin
      Res.Min_Priority:=The_Priority;
   end Set_Min_Priority;

   ------------------------------------
   -- min_priority --
   ------------------------------------

   function Min_Priority
     (Res : Processing_Resource)
     return Priority
   is
   begin
      return Res.Min_Priority;
   end Min_Priority;

   ------------------------------------
   -- Max_Any_Priority --
   ------------------------------------

   function Max_Any_Priority (Res : Processing_Resource)
     return Priority
   is
   begin
      return Res.Max_Priority;
   end Max_Any_Priority;

end Mast.Processing_Resources;
