-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with Ada.Text_IO,Var_Strings, Named_Lists;
package Mast.Processing_Resources is

   type Processing_Resource is abstract tagged private;

   procedure Init (Res : in out Processing_Resource;
                   Name : Var_Strings.Var_String);

   function Name (Res : Processing_Resource )
                  return Var_Strings.Var_String;

   procedure Set_Max_Priority (Res : in out Processing_Resource;
                               The_Priority : Priority);
   function Max_Priority (Res : Processing_Resource)
                          return Priority;

   procedure Set_Min_Priority (Res : in out Processing_Resource;
                               The_Priority : Priority);
   function Min_Priority (Res : Processing_Resource)
                          return Priority;

   function Max_Any_Priority (Res : Processing_Resource)
                          return Priority;

   procedure Set_Speed_Factor (Res : in out Processing_Resource;
                               The_Speed_Factor : Processor_Speed);
   function Speed_Factor (Res : Processing_Resource)
                          return Processor_Speed;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Processing_Resource;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   type Processing_Resource_Ref is access Processing_Resource'Class;

   function Clone
     (Res : Processing_Resource)
     return  Processing_Resource_Ref is abstract;

   function Name (Res_Ref : Processing_Resource_Ref )
                  return Var_Strings.Var_String;

   package Lists is new Named_Lists
     (Element => Processing_Resource_Ref,
      Name    => Name);

   function Clone
     (The_List : in Lists.List)
     return Lists.List;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      The_List : in out Lists.List;
      Indentation : Positive);

private

   type Processing_Resource is abstract tagged
      record
         Name : Var_Strings.Var_String;
         Speed_Factor : Processor_Speed:=1.0;
         Max_Priority : Priority:=Priority'Last;
         Min_Priority : Priority:=Priority'First;
      end record;


end Mast.Processing_Resources;
