-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with MAST.IO;
package body MAST.Scheduling_Servers is

   use type Scheduling_Parameters.Sched_Parameters_Ref;
   use type Processing_Resources.Processing_Resource_Ref;

   Names_Length : constant Positive := 26;

   -----------
   -- Clone --
   -----------

   function Clone
     (The_List : in Lists.List)
     return Lists.List
   is
      The_Copy : Lists.List;
      Iterator : Lists.Iteration_Object;
      Srvr_Ref, Srvr_Ref_Copy : Scheduling_Server_Ref;
   begin
      Lists.Rewind(The_List,Iterator);
      for I in 1..Lists.Size(The_List) loop
         Lists.Get_Next_Item(Srvr_Ref,The_List,Iterator);
         Srvr_Ref_Copy:=Clone(Srvr_Ref.all);
         Lists.Add(Srvr_Ref_Copy,The_Copy);
      end loop;
      return The_Copy;
   end Clone;


   -----------
   -- Clone --
   -----------

   function Clone
     (Srvr : Fixed_Priority_Server)
     return  Scheduling_Server_Ref
   is
      Srvr_Ref : Scheduling_Server_Ref;
   begin
      Srvr_Ref := new Fixed_Priority_Server'(Srvr);
      if Srvr.Server_Sched_Parameters/= null then
         Fixed_Priority_Server(Srvr_Ref.all).Server_Sched_Parameters:=
           Scheduling_Parameters.Clone(Srvr.Server_Sched_Parameters.all);
      end if;
      return Srvr_Ref;
   end Clone;

   -----------------------------
   -- Get_Min_System_Priority --
   -----------------------------

   function Get_Min_System_Priority
     (The_List : Lists.List)
     return Priority
   is
      Iterator : Lists.Iteration_Object;
      Srvr_Ref : Scheduling_Server_Ref;
      Prio : Priority;
      Min_Prio : Priority:=Priority'Last;
   begin
      Lists.Rewind(The_List,Iterator);
      for I in 1..Lists.Size(The_List) loop
         Lists.Get_Next_Item(Srvr_Ref,The_List,Iterator);
         if Srvr_Ref.all not in Fixed_Priority_Server'Class then
            raise Incorrect_Object;
         end if;
         Prio:=Scheduling_Parameters.The_Priority
           (Scheduling_Parameters.Fixed_Priority_Parameters
            (Srvr_Ref.Server_Sched_Parameters.all));
         if Prio < Min_Prio then
            Min_Prio:=Prio;
         end if;
      end loop;
      return Min_Prio;
   end Get_Min_System_Priority;


   ----------
   -- Init --
   ----------

   procedure Init
     (Srvr : in out Scheduling_Server;
      Name : Var_Strings.Var_String)
   is
   begin
      Srvr.Name:=Name;
   end Init;

   ----------
   -- Name --
   ----------

   function Name (Srvr : Scheduling_Server)
                  return Var_Strings.Var_String is
   begin
      return Srvr.Name;
   end Name;

   ----------
   -- Name --
   ----------

   function Name
     (Srvr_Ref : Scheduling_Server_Ref)
      return Var_Strings.Var_String
   is
   begin
      return Srvr_Ref.Name;
   end Name;

   -----------
   -- Print --
   -----------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Fixed_Priority_Server;
      Indentation : Positive;
      Finalize    : Boolean:=False;
      Complete    : Boolean:=True)
   is
   begin
      Ada.Text_IO.Set_Col(File,Ada.Text_IO.Count(Indentation));
      if Complete then
         Ada.Text_IO.Put_Line(File,"Scheduling_Server (");
      else
         Ada.Text_IO.Put(File,"  (");
      end if;
      MAST.IO.Print_Arg
        (File,"Type",
         "Fixed_Priority",Indentation+3,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Name",
         IO.Name_Image(Res.Name),Indentation+3,Names_Length);
      if Res.Server_Sched_Parameters/=null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg(File,"Server_Sched_Parameters","",
                           Indentation+3,Names_Length);
         MAST.IO.Print_Separator(File,"");
         MAST.Scheduling_Parameters.Print
           (File,Res.Server_Sched_Parameters.all,Indentation+6);
      end if;
      if Res.Server_Processing_Resource/=null then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Server_Processing_Resource",
            IO.Name_Image(MAST.Processing_Resources.Name
                                  (Res.Server_Processing_Resource)),
            Indentation+3,Names_Length);
      end if;
      if Complete then
         MAST.IO.Print_Separator(File,MAST.IO.Comma,Finalize);
      else
         Ada.Text_IO.Put(File,")");
      end if;
   end Print;


   -----------
   -- Print --
   -----------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      The_List : in out Lists.List;
      Indentation : Positive)
   is
      Res_Ref : Scheduling_Server_Ref;
      Iterator : Lists.Index;
   begin
      Lists.Rewind(The_List,Iterator);
      for I in 1..Lists.Size(The_List) loop
         Lists.Get_Next_Item(Res_Ref,The_List,Iterator);
         Print(File,Res_Ref.all,Indentation,True);
         Ada.Text_IO.New_Line(File);
      end loop;
   end Print;

   --------------------------------
   -- Server_Processing_Resource --
   --------------------------------

   function Server_Processing_Resource
     (Srvr : Scheduling_Server)
      return MAST.Processing_Resources.Processing_Resource_Ref
   is
   begin
      return Srvr.Server_Processing_Resource;
   end Server_Processing_Resource;

   -----------------------------
   -- Server_Sched_Parameters --
   -----------------------------

   function Server_Sched_Parameters
     (Srvr : Scheduling_Server)
      return MAST.Scheduling_Parameters.Sched_Parameters_Ref
   is
   begin
      return Srvr.Server_Sched_Parameters;
   end Server_Sched_Parameters;

   ------------------------------------
   -- Set_Server_Processing_Resource --
   ------------------------------------

   procedure Set_Server_Processing_Resource
     (Srvr : in out Scheduling_Server;
      The_Server_Processing_Resource :
        MAST.Processing_Resources.Processing_Resource_Ref)
   is
   begin
      Srvr.Server_Processing_Resource:=The_Server_Processing_Resource;
   end Set_Server_Processing_Resource;

   ---------------------------------
   -- Set_Server_Sched_Parameters --
   ---------------------------------

   procedure Set_Server_Sched_Parameters
     (Srvr : in out Scheduling_Server;
      The_Server_Sched_Parameters :
        MAST.Scheduling_Parameters.Sched_Parameters_Ref)
   is
   begin
      Srvr.Server_Sched_Parameters:=The_Server_Sched_Parameters;
   end Set_Server_Sched_Parameters;

end MAST.Scheduling_Servers;
