-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

package body Mast.Systems is

   -----------
   -- Clone --
   -----------

   function Clone(The_System : System) return System
   is
      The_Copy : System;
   begin
      declare
         Incomplete : exception;
      begin
         raise Incomplete;
      end;
      -- Clone the basic objects.
      -- Internal pointers will continue pointing at the old system
      The_Copy:=
        (Processing_Resources =>
           Processing_Resources.Clone(The_System.Processing_Resources),
         Shared_Resources =>
           Shared_Resources.Clone(The_System.Shared_Resources),
         Operations =>
           Operations.Clone(The_System.Operations),
         Transactions =>
           Transactions.Clone(The_System.Transactions),
         Scheduling_Servers =>
           Scheduling_Servers.Clone(The_System.Scheduling_Servers));

      -- rearrange all the internal pointers in simple operations
      Operations.Adjust(The_Copy.Operations,The_Copy.Shared_Resources);

      -- rearrange all the internal pointers in scheduling servers
      --Scheduling_Servers.Adjust(The_Copy.Scheduling_Servers,
      --                          The_Copy.Processing_Resources);

      -- rearrange all the internal pointers in network drivers:

      -- rearrange all the internal pointers in transactions:
      --       timing requirements, links, event handlers

      return The_Copy;
   end Clone;


   ----------------------
   -- Clone_Operations --
   ----------------------

   function Clone_Operations(The_System : System) return System
   is
      The_Copy : System;
   begin
      -- Clone the basic objects.
      -- Internal pointers will continue pointing at the old system
      The_Copy:=
        (Processing_Resources => The_System.Processing_Resources,
         Shared_Resources     => The_System.Shared_Resources,
         Operations           => Operations.Clone(The_System.Operations),
         Transactions         => The_System.Transactions,
         Scheduling_Servers   => The_System.Scheduling_Servers);

      -- rearrange the internal pointers referencing operations
      Transactions.Adjust_Operations
        (The_Copy.Transactions,The_Copy.Operations);


      return The_Copy;
   end Clone_Operations;

   -----------
   -- Print --
   -----------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      The_System : in out System;
      Indentation : Positive:=1)
   is
   begin
      MAST.Processing_Resources.Print
        (File,The_System.Processing_Resources,Indentation);
      MAST.Shared_Resources.Print
        (File,The_System.Shared_Resources,Indentation);
      MAST.Operations.Print
        (File,The_System.Operations,Indentation);
      MAST.Scheduling_Servers.Print
        (File,The_System.Scheduling_Servers,Indentation);
      MAST.Transactions.Print
        (File,The_System.Transactions,Indentation);
   end Print;

   -------------------
   -- Print_Results --
   -------------------

   procedure Print_Results
     (File : Ada.Text_IO.File_Type;
      The_System : in out System;
      Indentation : Positive:=1)
   is
   begin
      MAST.Transactions.Print_Results
        (File,The_System.Transactions,Indentation);
   end Print_Results;


end Mast.Systems;
