-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with Ada.Text_IO,
  MAST.Processing_Resources,
  MAST.Shared_Resources,
  MAST.Operations,
  MAST.Transactions,
  MAST.Scheduling_Servers;

package Mast.Systems is

   type System is tagged record
      Processing_Resources : MAST.Processing_Resources.Lists.List;
      Shared_Resources     : MAST.Shared_Resources.Lists.List;
      Operations           : MAST.Operations.Lists.List;
      Transactions         : MAST.Transactions.Lists.List;
      Scheduling_Servers   : MAST.Scheduling_Servers.Lists.List;
   end record;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      The_System : in out System;
      Indentation : Positive:=1);

   procedure Print_Results
     (File : Ada.Text_IO.File_Type;
      The_System : in out System;
      Indentation : Positive:=1);

   function Clone (The_System : System) return System;
   -- The Clone operation creates a copy of the whole system,
   -- but not including any results

   function Clone_Operations (The_System : System) return System;
   -- The Clone_Operations creates a copy of the system with all
   -- the operations cloned, and the rest of the objects shared among
   -- the two systems, through common pointers

end Mast.Systems;
