-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with Mast.IO,Var_Strings;
use Var_Strings;

package body Mast.Transactions.Results is

   ---------------
   -- Print     --
   ---------------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      The_List : in out Results_List;
      Indentation : Positive:=1;
      Finalize    : Boolean:=False)
   is
      Res : Transaction_Result;
      Trans_Ref : Transaction_Ref;
      Iterator : Results_Lists.Iteration_Object;
   begin
      Ada.Text_IO.Set_Col(File,Ada.Text_IO.Count(Indentation));
      Ada.Text_IO.Put_Line(File,"Transaction-Specific Slacks =>");
      Ada.Text_IO.Set_Col(File,Ada.Text_IO.Count(Indentation+2));
      Ada.Text_IO.Put(File,"(");
      Results_Lists.Rewind(Results_Lists.List(The_List),Iterator);
      for I in 1..Results_Lists.Size(Results_Lists.List(The_List)) loop
         Results_Lists.Get_Next_Item
           (Trans_Ref,Res,Results_Lists.List(The_List),Iterator);
         Ada.Text_IO.Set_Col(File,Ada.Text_IO.Count(Indentation+3));
         Ada.Text_IO.Put
           (File,To_String(Name(Trans_Ref))&" => "&
            IO.Slack_Image(Res.Slack));
         if I < Results_Lists.Size(Results_Lists.List(The_List)) then
            Ada.Text_IO.Put_Line(File,",");
         else
            Ada.Text_IO.Put_Line(File,")");
         end if;
      end loop;
   end Print;


   ---------------
   -- Set_Slack --
   ---------------

   procedure Set_Slack
     (Trans_Ref : Transaction_Ref;
      Slack : Float;
      The_List : in out Results_List)
   is
      Res : Transaction_Result;
   begin
      if Results_Lists.Exists(Trans_Ref,Results_Lists.List(The_List)) then
         Res:=Results_Lists.Item(Trans_Ref,Results_Lists.List(The_List));
         Res.Slack:=Slack;
         Results_Lists.Update(Trans_Ref,Res,Results_Lists.List(The_List));
      else
         Res.Slack:=Slack;
         Results_Lists.Add_Or_Update
           (Trans_Ref,Res,Results_Lists.List(The_List));
      end if;
   end Set_Slack;

   --------------
   -- Slack_Of --
   --------------

   function Slack_Of
     (Trans_Ref : Transaction_Ref;
      The_List : Results_List)
      return Float
   is
      Res : Transaction_Result;
   begin
      Res:=Results_Lists.Item(Trans_Ref,Results_Lists.List(The_List));
      return Res.Slack;
   end Slack_Of;

end Mast.Transactions.Results;

