-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000                          --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@ctr.unican.es                 --
--          Jose Javier Gutierrez  gutierjj@ctr.unican.es            --
--          Jose Carlos Palencia   palencij@ctr.unican.es            --
--          Jose Maria Drake       drakej@ctr.unican.es              --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with MAST.Results,MAST.Transactions,MAST.Graphs,MAST.Graphs.Links,
  MAST.Events,MAST.Systems,MAST.Timing_Requirements,Ada.Text_IO,
  Mast.Graphs.Event_Handlers;
use MAST.Results,MAST.Transactions,MAST.Graphs,MAST.Graphs.Links,MAST.Events,
  Ada.Text_IO,MAST.Timing_Requirements;

procedure MAST_Put_Results (The_System : in out MAST.Systems.System) is

   Trans_Ref : Transaction_Ref;
   Lnk_Ref : Link_Ref;
   Res_Ref : Result_Ref;
   The_Event : Event_Ref;
   Atr : Timing_Requirement_Ref;
   Is_Met : Boolean;
   Trans_Iterator : Mast.Transactions.Lists.Index;
   Link_Iterator, Ext_Link_Iterator : Mast.Transactions.Link_Iteration_Object;
begin
   MAST.Transactions.Lists.Rewind(The_System.Transactions,Trans_Iterator);
   for I in 1..MAST.Transactions.Lists.Size(The_System.Transactions) loop
      MAST.Transactions.Lists.Get_Next_Item
        (Trans_Ref,The_System.Transactions,Trans_Iterator);
      MAST.Transactions.Rewind_Internal_Event_Links
        (Trans_Ref.all,Link_Iterator);
      for J in 1..Num_Of_Internal_Event_Links(Trans_Ref.all) loop
         Get_Next_Internal_Event_Link(Trans_Ref.all,Lnk_Ref,Link_Iterator);
         if Input_Event_Handler(Lnk_Ref.all)/=null and then
           Input_Event_Handler(Lnk_Ref.all).all in
           MAST.Graphs.Event_Handlers.Activity'Class
         then
            Res_Ref  := Create_Timing_Result(Lnk_Ref);
            -- worst case parameters
            Set_Worst_Local_Response_Time
              (Timing_Result(Res_Ref.all),100.0);
            Set_Blocking_Time
              (Timing_Result(Res_Ref.all),33.0);
            MAST.Transactions.Rewind_External_Event_Links
              (Trans_Ref.all,Ext_Link_Iterator);
            for I in 1..Num_Of_External_Event_Links(Trans_Ref.all) loop
               Get_Next_External_Event_Link
                 (Trans_Ref.all,Lnk_Ref,Ext_Link_Iterator);
               The_Event:=Event_Of(Lnk_Ref.all);
               Set_Worst_Global_Response_Time
                 (Timing_Result(Res_Ref.all),The_Event,120.0);
               Set_Best_Global_Response_Time
                 (Timing_Result(Res_Ref.all),The_Event,40.0);
               Set_Jitter
                 (Timing_Result(Res_Ref.all),The_Event,5.0);
            end loop;
            -- simulation parameters
            Reset_Local_Miss_Ratio(Timing_Result(Res_Ref.all),56.0);
            Reset_Local_Miss_Ratio(Timing_Result(Res_Ref.all),160.0);
            for T in 10..100 loop
               Record_Local_Simulation_Time
                 (Timing_Result(Res_Ref.all),Mast.Time(T));
            end loop;
            MAST.Transactions.Rewind_External_Event_Links
              (Trans_Ref.all,Ext_Link_Iterator);
            for I in 1..Num_Of_External_Event_Links(Trans_Ref.all) loop
               Get_Next_External_Event_Link
                 (Trans_Ref.all,Lnk_Ref,Ext_Link_Iterator);
               The_Event:=Event_Of(Lnk_Ref.all);
               Reset_Global_Miss_Ratio
                 (Timing_Result(Res_Ref.all),230.0,The_Event);
               Reset_Global_Miss_Ratio
                 (Timing_Result(Res_Ref.all),320.0,The_Event);
               for T in 210..300 loop
                  Record_Global_Simulation_Time
                    (Timing_Result(Res_Ref.all),The_Event,Mast.Time(T));
               end loop;
            end loop;
            for T in 310..400 loop
               Record_Global_Simulation_Time
                 (Timing_Result(Res_Ref.all),Mast.Time(T));
            end loop;
            Atr:=Link_Timing_Requirements
              (Regular_Link(Lnk_Ref.all));
            if Atr/=null
            then
               Print(Standard_Output,Atr.all,1);
               begin
                  Check(Link_Timing_Requirements
                        (Regular_Link(Lnk_Ref.all)).all,
                        Timing_Result(Res_Ref.all),
                        Is_Met);
                  if Is_Met then
                     Put_Line("Requirement met");
                  else
                     Put_Line("...Missed");
                  end if;
               exception
                  when Inconclusive =>
                     Put_Line("...Inconclusive");
               end;
               Print(Standard_Output,Res_Ref.all,5);
            end if;
         end if;
      end loop;
   end loop;
end MAST_Put_Results;
